unit Filter;

(**********************************************************)

                        interface

(**********************************************************)

uses FilterNode,ComCtrls, Classes,Controls,StdCtrls,Contnrs;
type

 TFilter= class(TObject)
 private
      //FList holds all verteces of filter tree
      //this information duplicates property
      // FTreeView.Items[i].data
      FList:TList;

      //current filtered string
      FString:String;

      //true when input is terminated
      FTerminated:boolean;

      //holds current node tree
      //during input
      //the node's type is always  ntStateNode
      FCurrentNode:TTreeNode;


      //depending on FCurrentNode determines
      //which of it's springs *c* belongs to
      function getActivebranch(c:char):TTreeNode;

      //both branch and state nodes can be terminal
      function isNodeTerminal(node:TTreeNode):boolean;

      //accepts set *s* of chars and determines
      //whether the intersection of *s* and all
      //sets of verteces at level of *subling*
      //is empty.  subling can be any at level
      function alreadyIncluded(const s:TCharSet; const subling:TTreeNode):boolean;

      //accepts branch-vertex(any) and returns it's state-vertex
      function getStateVertex(const node:TTreeNode):TTreeNode;

      function TreeNodeToFilterNode(node:TTreeNode):TFilterNode;

      
      procedure OnKeyPress(Sender: TObject; var Key: Char);
      procedure OnChange(Sender: TObject);
      procedure OnKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);

     //accpets  state-vertex and returns union
     //of sets of all it's branches
     //if next branch in turn is nonterminal it is pushed to *st*
     function findUnion(const stateNode:TTreeNode; var st:TStack):TCharSet;
 public



      FEdit:TEdit;

      //tip for user with current set
      FLabel:TLabel;
      FTreeView:TTreeView;


      function terminated:boolean;

      constructor Create(treeOwner:TComponent; treePar:TWinControl);
      destructor Destroy; override;


      //add set *s* to branch or state-vertex
      procedure addSet   (const s:TCharSet);

      //add to branch(i. e. at the same level)
      //or state-vertex one branch
      procedure addbranch(const s:TCharSet);

      //add to selected branch hild (state-vetrex)
      procedure addState (const s:TCharSet);

      //removes FTerminated,  begins input from scratch
      procedure resetInput;

      procedure init(Sender: TObject);

      //the folowing is tested:
      //if state-vertex have at least one branch
      //then all it's branches must exhaust it's set
      function checkTree:Boolean;


      procedure saveToStream(s:TStream);
      procedure loadFromStream(s:TStream);

 end;

(**********************************************************)

                    implementation

(**********************************************************)
uses DateUtils, Dialogs,ExtCtrls, SysUtils,windows;

procedure TFilter.OnKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);
begin
  if Key=VK_BACK then
       resetInput;
end;

function TFilter.TreeNodeToFilterNode(node:TTreeNode):TFilterNode;
begin
  assert(assigned(node));
  result:=TFilterNode(node.data);
end;


function TFilter.terminated:boolean;
begin
  result:=FTerminated;
end;


procedure TFilter.saveToStream(s:TStream);
var
    n:integer;
    nd:TTreeNode;
begin


   //**************   n **************
   n:=FList.Count;
   s.Write(n,sizeof(n));

   //**************   component **************
   s.WriteComponent(FTreeView);


  //**************   items **************
  nd:=FTreeView.TopItem;
  while nd<>nil do
  begin
      TreeNodeToFilterNode(nd).saveToStream(s);
      nd:=nd.GetNext;
  end;
       
end;

procedure TFilter.loadFromStream(s:TStream);
var i,n:Integer;
    nd:TTreeNode;
begin
  assert(assigned(FTreeView));
  assert(FTreeView.Items.Count=0);


  //**************   n  **************
  s.Read(n,sizeof(n));

  //**************   component  **************
  s.ReadComponent(FTreeView);

  assert(n>0);
  assert(FList.count=0);

  //**************   items **************
  nd:=FTreeView.TopItem;
 
      for i:=0 to n-1 do
      begin
          FList.Add(TFilterNode.create(ntStateNode));
          TFilterNode(FList[i]).loadFromStream(s);
          nd.Data:= FList[i];
          nd:=nd.GetNext;
          if (nd=nil)and(i<>n-1) then
              raise Exception.Create('i='+IntToStr(i)+'n='+IntToStr(n));
      end;


  assert(FTreeView.Items.count=FList.Count);

  checkTree;
  resetInput;

end;

constructor TFilter.Create(treeOwner:TComponent; treePar:TWinControl);
begin


  FTreeView:=TTreeView.Create(treeOwner);
  FTreeView.Parent:=treePar;
  FTreeView.Visible:=false;


  FLabel:=TLabel.Create(treeOwner);
  FLabel.Parent:=treePar;

  FLabel.Left:=15;
  FLabel.Top:=15;


  FEdit:=TEdit.Create(treeOwner);
  FEdit.Parent:=treePar;

  FEdit.Left:=15;
  FEdit.Top:=FLabel.Top+20;

  FEdit.OnKeyPress:=OnKeyPress;
  FEdit.OnChange:=OnChange;
  FEdit.OnKeyDown:=OnKeyDown;



 


  FList:=TList.Create;
  ///////////////////////////////////////
end;
{----------------------------------------------------------}
procedure TFilter.init(Sender: TObject);
var firstNode:TFilterNode;
    s:string;
begin
  (Sender as TTimer).Enabled:=false;

  
  {
    case   1:
              component is created first time
              FList.Count=0 => it's need to create
              top vertex
    case   2:
              component was created, it's loadFromStream method
              was called => FList.Count>0
              first vertex was read from stream as others

  }
  assert(FTreeView.Items.count=FList.Count); // at the end of  TFilter.loadFromStream
                                             // OK, but in Delphi 2006 violates here !
                                             // in Delphi 7 everything OK
  if FList.Count>0 then
    exit;

  firstNode:=TFilterNode.create(ntStateNode);
  FList.Add(firstNode);
  firstNode.getCaption(s);

  FTreeView.Items.AddObjectFirst(nil,s,firstNode);

  FCurrentNode:=FTreeView.TopItem;



  FTreeView.Width:=270;
  FTreeView.Height:=360;

  resetInput;

end;
{----------------------------------------------------------}
function TFilter.isNodeTerminal(node:TTreeNode):boolean;
begin
  assert(node<>nil);
  case TreeNodeToFilterNode(node).FNodeType of
    ntStateNode:
            result:=(node.getNextSibling=nil);
    ntbranchNode:
          result:=(node.Count=0);
  end;
end;
{----------------------------------------------------------}
function TFilter.findUnion(const stateNode:TTreeNode; var st:TStack):TCharSet;
var union, setOfbranch:TCharSet;
    walker:TTreeNode;
begin
   assert(not isNodeTerminal(stateNode));
   assert(TreeNodeToFilterNode(stateNode).FNodeType=ntStateNode);

   union:=[];

   walker:=stateNode.getNextSibling; //Exists !
   while walker<>nil do
   begin
       if not isNodeTerminal(walker) then
          st.Push(walker.getFirstChild);
       TreeNodeToFilterNode(walker).getValidChars(setOfbranch);
       union:=union+setOfbranch;
       walker:=walker.getNextSibling;
   end;

   Result:=union;
end;
{----------------------------------------------------------}
function TFilter.checkTree:Boolean;
var st:TStack;
    stateNode:TTreeNode;
    stateSet:TCharSet;

begin
  Result:=true;
  if FTreeView.Items.Count=0 then
  begin
    Result:=False;
    ShowMessage('error in tree');
    exit;
  end;
  st:=TStack.Create;
       st.Push(FTreeView.TopItem);
       while st.AtLeast(1) do
       begin
            stateNode:=TTreeNode(st.pop);
            if not isNodeTerminal(stateNode) then
            begin
                 TreeNodeToFilterNode(stateNode).getValidChars(stateSet);
                 if stateSet<>findUnion(stateNode,st) then
                 begin
                     ShowMessage('error in tree');
                     Result:=false;
                     break;
                 end;
            end;
       end;
  st.Free;
end;
{----------------------------------------------------------}
function TFilter.getActivebranch(c:char):TTreeNode;
var walker:TTreeNode;
    s:TCharSet;
begin
  assert(TreeNodeToFilterNode(FCurrentNode).FNodeType=ntStateNode);
  assert(not isNodeTerminal(FCurrentNode));

  walker:=FCurrentNode.getNextSibling; //<>nil  ; it's first branch

  Result:=nil;
  while walker<>nil do
  begin
      TreeNodeToFilterNode(walker).getValidChars(s);
      if c in s then
      begin
          Result:=walker;
          exit;
      end;
      walker:=walker.getNextSibling;
  end;
end;
{----------------------------------------------------------}
procedure TFilter.resetInput;
var str:string;
begin

     FString:='';
     FTerminated:=false;
     FCurrentNode:= FTreeView.TopItem;
     FEdit.Clear;

     TreeNodeToFilterNode(FCurrentNode).getCaption(str);
     FLabel.Caption:=str;
end;
{----------------------------------------------------------}
destructor TFilter.Destroy;
var i:Integer;
begin
  for i:=0 to pred(FList.Count) do
      TFilterNode(FList[i]).Free;
  FList.Free;
end;
{----------------------------------------------------------}
procedure TFilter.OnKeyPress(Sender: TObject; var Key: Char);
var
    validChars:TCharSet;
    str:string;
    activeBranch:TTreeNode;
begin

  if FTerminated then exit;

  if not isNodeTerminal(FCurrentNode) then
  begin
       activeBranch:=getActivebranch(Key);


       if activeBranch=nil then exit;

       FString:=FString+Key;
       if not isNodeTerminal(activeBranch) then
       begin
          //going to next state-vertex
          //getFirstChild can be state-vertex and only it.
          FCurrentNode:=activeBranch.getFirstChild;
          TreeNodeToFilterNode(FCurrentNode).getCaption(str);
          FLabel.Caption:=str;
       end
           else
       begin
          FLabel.Caption:='Terminated';
          FTerminated:=true;
       end

  end
      else
  begin //we are now in terminal node and ready to terminate input
       TreeNodeToFilterNode(FCurrentNode).getValidChars(validChars);
       if Key in validChars then
       begin
            FLabel.Caption:='Terminated';
            FString:=FString+Key;
            FTerminated:=true;
       end;
  end;
  OnChange(Sender);
end;
{----------------------------------------------------------}
procedure TFilter.OnChange(Sender: TObject);
begin
   FEdit.Text:=FString;
end;
{----------------------------------------------------------}
function TFilter.getStateVertex(const node:TTreeNode):TTreeNode;
var walker:TTreeNode;
begin
  walker:=node;
  result:=nil;
  while walker<>nil do
  begin
       result:=walker;
       walker:=walker.getPrevSibling;
  end;
end;
{----------------------------------------------------------}
function TFilter.alreadyIncluded(const s:TCharSet; const subling:TTreeNode):boolean;
var
    walker:TTreeNode;
    stateNode:TTreeNode;
begin
    result:=false;
    walker:=subling;

    stateNode:=getStateVertex(subling);
    while walker<>nil do
    begin
        //if walker now state then  getPrevSibling=nil
        if walker.getPrevSibling=nil then break;
        if TreeNodeToFilterNode(walker).anyCommon(s) then
        begin
            result:=true;
            exit;
        end;
        walker:=walker.getPrevSibling;
    end;

    walker:=subling.getNextSibling;
    while walker<>nil do
    begin
        if walker=stateNode then Continue;
        if TreeNodeToFilterNode(walker).anyCommon(s) then
        begin
            result:=true;
            exit;
        end;
        walker:=walker.getNextSibling;
    end;
end;
{----------------------------------------------------------}
procedure TFilter.addSet(const s:TCharSet);
var NodeToAdd:TFilterNode;
    str:string;
    stateNodeSet:TCharSet;
begin
  NodeToAdd:=TreeNodeToFilterNode(FTreeView.Selected);

  //if selected state-node
  if (getStateVertex( FTreeView.Selected)= FTreeView.Selected) then
  begin
      //no duplicates
      NodeToAdd.addSet(s);
      NodeToAdd.getCaption(str);
       FTreeView.Selected.Text:=str;
  end
  else //non-state node
  begin
     TreeNodeToFilterNode(getStateVertex( FTreeView.Selected)).getValidChars(stateNodeSet);
     if not ( s<=stateNodeSet) then exit;
     if alreadyIncluded(s, FTreeView.Selected) then exit;
     NodeToAdd.addSet(s);    //no duplicates
     NodeToAdd.getCaption(str);
      FTreeView.Selected.Text:=str;
      FTreeView.Refresh;
  end;
end;
{----------------------------------------------------------}
procedure TFilter.addbranch(const s:TCharSet);
var newNode:TFilterNode;
    str:string;
    stateNodeSet:TCharSet;
begin
     //save set of FTreeView.Selected to stateNodeSet
     TreeNodeToFilterNode(getStateVertex( FTreeView.Selected)).getValidChars(stateNodeSet);

     //behave oneself  :))
     if not (s <= stateNodeSet) then exit;
     if alreadyIncluded(s, FTreeView.Selected) then exit;


     newNode:=TFilterNode.create(ntbranchNode);
     FList.Add(newNode);
     newNode.addSet(s);    //no duplicates
     newNode.getCaption(str);
     FTreeView.Items.AddObject( FTreeView.Selected,str,newNode);
end;
{----------------------------------------------------------}
procedure TFilter.addState (const s:TCharSet);
var newNode:TFilterNode;
    str:string;
    stateNodeSet:TCharSet;
    selected:TTreeNode;
begin
     selected:= FTreeView.Selected;
     if (TreeNodeToFilterNode(selected).FNodeType=ntStateNode ) then exit;

     if selected= FTreeView.TopItem then exit;
     
     TreeNodeToFilterNode(getStateVertex(selected)).getValidChars(stateNodeSet);

     // branch can have maximum one state-vertex
     if selected.Count<>0 then exit;

     newNode:=TFilterNode.create(ntStateNode);
     FList.Add(newNode);
     newNode.addSet(s);    //no duplicates
     newNode.getCaption(str);
     FTreeView.Items.AddChildObject( FTreeView.Selected,str,newNode);

end;
{----------------------------------------------------------}


end.
