unit wizardForm;

interface

uses Classes, Forms, Controls, Buttons, DesignIntf,
  StdCtrls, ExtCtrls,DesignEditors, Dialogs, FilteredEdit;



type
  TwizForm = class(TForm)
    addChar: TButton;
    addCharEdit: TEdit;
    lowBoundEdit: TEdit;
    HighBoundEdit: TEdit;
    addRange: TButton;
    clearBtn: TButton;
    RadioGroup1: TRadioGroup;
    procedure clearBtnClick(Sender: TObject);
    procedure addCharClick(Sender: TObject);
    procedure addRangeClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FFilteredEdit:TFilteredEdit;
  public

  end;


  TFilterEditor = class(TComponentEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

  procedure Register;

var wizForm:TwizForm;


implementation

{$R *.dfm}

uses  FilterNode;

procedure Register;
begin
  RegisterComponents('ProtectedInput',[TFilteredEdit]);
  RegisterComponentEditor(TFilteredEdit, TFilterEditor);
end;


procedure TFilterEditor.Edit;
begin
     if not assigned(wizForm) then
     begin
        wizForm:=TwizForm.Create(Application);
     end;

    wizForm.FFilteredEdit:=(component as TFilteredEdit);

    (component as TFilteredEdit).FFilter.FTreeView.Visible:=true;

    //the owner of FFilter.FTreeView is FilteredEdit
    //but when the component editor starts it becomes
    //temporary the parent of FFilter.FTreeView
    //when editor is closed, the parent becomes FilteredEdit
    (component as TFilteredEdit).FFilter.FTreeView.Parent:=wizForm;
    wizForm.Show;
end;


function TFilterEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TFilterEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then
    Result := 'start editor'
  else Result := '';
end;

procedure TFilterEditor.ExecuteVerb(Index: Integer);
begin
  if Index<>0 then exit;
  Edit;
end;





procedure TwizForm.clearBtnClick(Sender: TObject);
begin
   FFilteredEdit.FFilter.checkTree;
   FFilteredEdit.FFilter.resetInput;
end;

procedure TwizForm.addCharClick(Sender: TObject);
var newSet:TCharSet;
begin
       if FFilteredEdit.FFilter.FTreeView.Selected=nil then
            exit;

        if  length(addCharEdit.Text)<>1 then
        begin
            ShowMessage('invalid char');
            exit;
        end;

       
        newSet:=[addCharEdit.Text[1]];
 

        case TWhereToAdd(RadioGroup1.ItemIndex) of
           wtaToNode:
              FFilteredEdit.FFilter.addSet(newSet);
           wtaAsSibling:
              FFilteredEdit.FFilter.addBranch(newSet);
           wtaAsChild:
               FFilteredEdit.FFilter.addState(newSet);
        end;
end;

procedure TwizForm.addRangeClick(Sender: TObject);
var
    includedSet:TCharSet;
    c:char;
begin
       if   FFilteredEdit.FFilter.FTreeView.Selected=nil then
            exit;


       includedSet:=[];
       for c:=lowBoundEdit.Text[1] to highBoundEdit.Text[1] do
           include(includedSet,c);

       if (length(lowBoundEdit.Text)<>1)or(length(highBoundEdit.Text)<>1) then
       begin
           ShowMessage('invalid range');
           exit;
       end;

       if (lowBoundEdit.Text[1]>=highBoundEdit.Text) then
       begin
           ShowMessage('check range bounds');
           exit;
       end;

       case TWhereToAdd(RadioGroup1.ItemIndex) of
           wtaToNode:
                FFilteredEdit.FFilter.addSet(includedSet);
           wtaAsSibling:
               FFilteredEdit. FFilter.addbranch(includedSet);
           wtaAsChild:
               FFilteredEdit. FFilter.addState(includedSet);
       end;
end;


procedure TwizForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FFilteredEdit.FFilter.FTreeView.Parent:=FFilteredEdit;
  FFilteredEdit.FFilter.FTreeView.Visible:=false;
end;



end.
